package tutorial.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.JoinTable;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import org.apache.commons.logging.*;

/**
 * <b>Created:</b> Mar 1, 2008<br>
 * <b>Title:</b> Project<br>
 * <b>Description:</b><br>
 * <b>Copyright:</b> Copyright (c) 2008<br>
 * @author MTyson
 * <p>
 * @todo add class overview here
 */
@Entity
public class Project implements ManagedObject {
    private static final transient Log log = LogFactory.getLog(tutorial.model.Project.class);
    @Column (name="project_id")
    @Id
    @GeneratedValue(strategy = GenerationType.TABLE)
    private Long projectId;
    @Column (length = 50)
    private String name;
    @Column(name = "start_date")
    @Temporal(TemporalType.DATE)
    private Date startDate;
    
    @ManyToMany(cascade = {CascadeType.PERSIST,CascadeType.MERGE})
    @JoinTable(
            name="project_manager",
            joinColumns=@JoinColumn(name="project_id",
                    referencedColumnName="project_id"),
                    inverseJoinColumns=@JoinColumn(
                            name="manager_id",
                            referencedColumnName="manager_id")
    )
    private List<Manager> managers = new ArrayList<Manager>();
    
    /**default empty constructor*/
    public Project() {
    }
    
    public Long getId(){
        return this.getProjectId();
    }
    public String getLabel(){
        return this.getName();
    }
    
    public boolean equals(Object o){
        if (o != null){
            if (o instanceof Project){
                if (((Project)o).getProjectId().equals(this.getProjectId())){
                    return true;
                }
            }
        }
        return false;
    }
    public String toString(){
        return "Project: " + this.getProjectId() + " - " + this.getName() + ", managers: " + this.getManagers().size();
    }
    public Project(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public Long getProjectId() {
        return projectId;
    }

    public void setProjectId(Long projectId) {
        this.projectId = projectId;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }
    public List<Manager> getManagers() {
        if (log.isTraceEnabled()) { log.trace("managers count: " + managers.size()); }
        return managers;
    }
    public void setManagers(List<Manager> managers) {
        if (log.isInfoEnabled()) { log.info("Managers set: " + managers); }
        this.managers = managers;
    }

}
