package tutorial.dao;

import java.io.File;
import java.util.List;

import javax.sound.midi.MidiDevice.Info;

import org.springframework.test.jpa.AbstractJpaTests;

//import tutorial.service.CommonService;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.logging.*;

import com.sun.org.apache.bcel.internal.generic.IFNONNULL;

import tutorial.model.Developer;
import tutorial.model.ManagedObject;
import tutorial.model.Manager;
import tutorial.model.Project;
import tutorial.service.CommonService;
import tutorial.service.PathProxyService;
import tutorial.util.EntityPath;

/**
 * <b>Created:</b> Feb 27, 2008<br>
 * <b>Title:</b> DaoTest<br>
 * <b>Description:</b><br>
 * <b>Copyright:</b> Copyright (c) 2008<br>
 * @author MTyson
 * <p>
 * @todo add class overview here
 */

public class DaoTest extends AbstractJpaTests {
    private static final transient Log log = LogFactory.getLog(tutorial.dao.DaoTest.class);

    /**default empty constructor*/
    public DaoTest() {
        if (log.isWarnEnabled()) { log.warn("Constructor"); }
    }
    
    protected void onSetUpInTransaction() throws Exception {
        if (log.isWarnEnabled()) { log.warn("BEGIN onSetUpInTransaction()"); }
    }
    private CommonService commonService;
    private PathProxyService pathProxyService;

    private long JoeSmithId = 99999;

    public void setCommonService(CommonService commonService) {
       this.commonService = commonService;
    }

    protected String[] getConfigLocations() {
        File file = new File(".");
        try {
            if (log.isWarnEnabled()) { log.warn("LOCATION: " + file.getCanonicalPath()); }
        } catch (Exception e){
            
        }
        return new String[] { "file:../webapp/src/main/webapp/WEB-INF/applicationContext.xml" };
    }

    public void testProjectAndDev(){
        Manager man = new Manager("John");
        Project project = new Project("Project 1");
        List<Manager> projectManagers = project.getManagers();
        man = (Manager)commonService.save(man);
        projectManagers.add(man);
        project = (Project)commonService.save(project);
        
        Project projectRecovered = (Project)commonService.get(Project.class, project.getProjectId());
        assertTrue(projectRecovered.getName().equals("Project 1"));
        assertTrue(projectRecovered.getManagers().contains(man));
        
    }
    public void testDeveloper(){
        if (log.isTraceEnabled()) { log.trace("BEGIN testDeveloper()"); }
        Developer dev = new Developer("John");
        dev = (Developer)commonService.save(dev);
//        setComplete();
    }
    public void testPathProxy(){
        Manager man = new Manager("John");
        Project project = new Project("Project 1");
        List<Manager> projectManagers = project.getManagers();
        man = (Manager)commonService.save(man);
        projectManagers.add(man);
        project = (Project)commonService.save(project);
        
        Developer dev = (new Developer("test"));
        dev = (Developer)commonService.save(dev);
        
        EntityPath path = new EntityPath().setPathAsObjects(new ManagedObject[]{project, man});
        pathProxyService.addChild(path, dev);
        
        List<Object> children = pathProxyService.getPathChildren(path, Developer.class.getName());
        if (log.isInfoEnabled()) { log.info("children: " + ArrayUtils.toString(children)); }
        if (log.isInfoEnabled()) { log.info("dev: " + dev); }
        assertTrue(children.contains(dev));
        
        EntityPath emptyPath = new EntityPath().setPathAsObjects(new ManagedObject[]{new Project("test"), man});
        List<Object> noKids = pathProxyService.getPathChildren(emptyPath, Developer.class.getName());
        if (log.isInfoEnabled()) { log.info("noKids: " + noKids); }
        assertTrue(noKids.size() == 0);
//        setComplete();
    }
    
    public void testPathProxyCreate(){
        Manager man = new Manager("John");
        Project project = new Project("Project 1");
        List<Manager> projectManagers = project.getManagers();
        man = (Manager)commonService.save(man);
        projectManagers.add(man);
        project = (Project)commonService.save(project);
        
        Developer dev = (new Developer("test"));
        dev = (Developer)commonService.save(dev);
        
        pathProxyService.addChild(new EntityPath().setPathAsObjects(new ManagedObject[]{project, man}), dev);
//        setComplete();
    }

    public PathProxyService getPathProxyService() {
        return pathProxyService;
    }

    public void setPathProxyService(PathProxyService pathProxyService) {
        this.pathProxyService = pathProxyService;
    }
}
