package tutorial.webapp.backingbean;

import java.util.ArrayList;
import java.util.List;

import javax.faces.event.ActionEvent;
import javax.faces.model.SelectItem;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.logging.*;

import tutorial.dao.CommonDao;
import tutorial.model.Developer;
import tutorial.model.ManagedObject;
import tutorial.model.Manager;
import tutorial.model.Person;
import tutorial.model.Task;
import tutorial.service.CommonService;
import tutorial.service.PathProxyService;
import tutorial.util.Constant;
import tutorial.util.EntityPath;

/**
 * <b>Created:</b> Apr 5, 2008<br>
 * <b>Title:</b> PersonBean<br>
 * <b>Description:</b><br>
 * <b>Copyright:</b> Copyright (c) 2008<br>
 * @author MTyson
 * <p>
 * @todo add class overview here
 */

public class PersonBean extends ManagedObjectBackingBean {
    private static final transient Log log = LogFactory.getLog(tutorial.webapp.backingbean.PersonBean.class);

    private String firstName;
    private String type;
    private CommonService commonService;
    private SessionBean sessionBean;
    private PathProxyService pathProxyService;
    
    public PathProxyService getPathProxyService() {
        return pathProxyService;
    }

    public void setPathProxyService(PathProxyService pathProxyService) {
        this.pathProxyService = pathProxyService;
    }

    public String addNew(){
        if (log.isInfoEnabled()) { log.info("Adding a new person: " + firstName + " | " + type); }
        
        boolean flushRequired = false;
        
        Class clazz = null;
        try {
             clazz = Class.forName(type);
        } catch (Exception e){
            throw new RuntimeException("Unknown type: " + type, e);
        }
        
        Person newP = null;
        try {
            newP = (Person)clazz.newInstance();
        } catch (Exception e){
            throw new RuntimeException("Problem instantiating obecjt: " + newP, e);
        }
        newP.setFirstName(firstName);
        newP = (Developer)commonService.save(newP);
        flushRequired = true;
        
        if (flushRequired){
            commonService.flush();
        }
        return "home";
    }
    
    /**
     * Notice that the next four method are very similar.  Handling them in a generic way as you have more of the same 
     * requirement would be best.  If JSF/EL would support parameters, that would make this much easier.  As it stands,
     * you could pass back the objects types you are interested in either as a request param or using a map-hack to
     * pass params.
     * @return
     */
    public List getDevelopersOnSelectedManager(){
        assert(Manager.class.getName().equals(sessionBean.getSelectionType()));
        String nodeString = sessionBean.getSelectionNodeString();
        if (log.isTraceEnabled()) { log.trace("nodeString: " + nodeString); }
        EntityPath path = this.getNewPath().setPathAsString(nodeString);
        if (log.isTraceEnabled()) { log.trace("path: " + ArrayUtils.toString(path)); }
        List devs = pathProxyService.getPathChildren(path, Developer.class.getName());
        return devs;
    }
    
    public void setDevelopersOnSelectedManager(List devsFromUi){
        assert(Manager.class.getName().equals(sessionBean.getSelectionType()));
        String nodeString = sessionBean.getSelectionNodeString();
        EntityPath path = this.getNewPath().setPathAsString(nodeString);
        if (log.isTraceEnabled()) { log.trace("path: " + ArrayUtils.toString(path)); }
        List existingO = pathProxyService.getPathChildren(path, Developer.class.getName());
        
        List<Developer> actualDevs = new ArrayList<Developer>();
        
        for (Object o : existingO){
            Developer existingDev = (Developer)o;
            if (!devsFromUi.contains(existingDev)){
                pathProxyService.removeChild(path, existingDev);
            }
        }
        
        for (Object o : devsFromUi){
            Developer newDev = (Developer)o;
            if (!existingO.contains(newDev)){
                pathProxyService.addChild(path, newDev);
            }
        }
    }
    
    public List getTasksOnSelectedDeveloper(){
        assert(Developer.class.getName().equals(sessionBean.getSelectionType()));
        String nodeString = sessionBean.getSelectionNodeString();
        if (log.isTraceEnabled()) { log.trace("nodeString: " + nodeString); }
        EntityPath path = this.getNewPath().setPathAsString(nodeString);
        if (log.isTraceEnabled()) { log.trace("path: " + ArrayUtils.toString(path)); }
        List o = pathProxyService.getPathChildren(path, Task.class.getName());
        return o;
    }
    
    public void setTasksOnSelectedDeveloper(List tasksFromUi){
        assert(Manager.class.getName().equals(sessionBean.getSelectionType()));
        String nodeString = sessionBean.getSelectionNodeString();
        EntityPath path = this.getNewPath().setPathAsString(nodeString);
        if (log.isTraceEnabled()) { log.trace("path: " + ArrayUtils.toString(path)); }
        List existingO = pathProxyService.getPathChildren(path, Task.class.getName());
        
        List<Developer> actualDevs = new ArrayList<Developer>();
        
        for (Object o : existingO){
            if (!tasksFromUi.contains(o)){
                pathProxyService.removeChild(path, (ManagedObject)o);
            }
        }
        
        for (Object o : tasksFromUi){
            if (!existingO.contains(o)){
                pathProxyService.addChild(path, (ManagedObject)o);
            }
        }
    }
    
    public EntityPath getNewPath(){
        // Spring injected
        return null;
    }
    
    public List getAllManagers(){
        return this.getMoAsSelectItems(commonService.getAll(Manager.class));
    }
    
    public List getAllDevelopers(){
        return this.getMoAsSelectItems(commonService.getAll(Developer.class));
    }
    
    public String getType() {
        return type;
    }

    public void setType(String type) {
        this.type = type;
    }

    /**default empty constructor*/
    public PersonBean() {
    }
    public String getFirstName() {
        return firstName;
    }
    public void setFirstName(String firstName) {
        this.firstName = firstName;
    }

    public CommonService getCommonService() {
        return commonService;
    }

    public void setCommonService(CommonService commonService) {
        this.commonService = commonService;
    }

    public SessionBean getSessionBean() {
        return sessionBean;
    }

    public void setSessionBean(SessionBean sessionBean) {
        this.sessionBean = sessionBean;
    }

}
