package tutorial.webapp.backingbean;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.faces.event.ValueChangeEvent;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.logging.*;

import tutorial.service.CommonService;
import tutorial.util.EntityPath;

/**
 * <b>Created:</b> Apr 12, 2008<br>
 * <b>Title:</b> SessionBean<br>
 * <b>Description:</b><br>
 * <b>Copyright:</b> Copyright (c) 2008<br>
 * @author MTyson
 * <p>
 * @todo add class overview here
 */

public class SessionBean {
    private static final transient Log log = LogFactory.getLog(tutorial.webapp.backingbean.SessionBean.class);

    private CommonService commonService;
    private Object currentSelection;
    private String selectionType;
    private String selectionId;
    private String selectionNodeString;
    private EntityPath selectionEntityPath;

    public String getSelectionId() {
        return selectionId;
    }

    public void setSelectionId(String selectionId) {
        this.selectionId = selectionId;
    }

    /**default empty constructor*/
    public SessionBean() {
    }

    public String persistCurrentSelection(){
        if (log.isInfoEnabled()) { log.info("BEGIN persistCurrentSelection()"); }
        try {
            currentSelection = commonService.save(currentSelection);
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("Changes saved"));
        } catch (Exception e){
            FacesContext.getCurrentInstance().addMessage(null, new FacesMessage("There was a problem saving: " + e.getMessage()));
        }
        if (log.isInfoEnabled()) { log.info("currentSelection has been saved: " + currentSelection); }
        return "home";
    }
    public String getSelectionType() {
        return selectionType;
    }
    
    public Object getCurrentSelection(){
        if (log.isInfoEnabled()) { log.info("BEGIN getSelection()"); }
        if (log.isTraceEnabled()) { log.trace("currentSelection is: " + currentSelection); }
        FacesContext context = FacesContext.getCurrentInstance();
       
        String selectionType = (String)context.getExternalContext().getRequestParameterMap().get("selectionType");
        String selectionNodeString = (String)context.getExternalContext().getRequestParameterMap().get("selectionNodeString"); 
        
        if (log.isTraceEnabled()) { log.trace("selectionType: " + selectionType + " | selectionId: " + selectionId + " | selectionNodeString: " + selectionNodeString); }
        
        if (StringUtils.isBlank(selectionType) || StringUtils.isBlank(selectionNodeString)){
            if (log.isTraceEnabled()) { log.trace("Selection has remained the same, as type and ID are null."); }
        } else {
            // Get id off the nodeString
            String[] nodeStringSplit = selectionNodeString.split("\\^"); 
            String selectionId = nodeStringSplit[nodeStringSplit.length - 1];
            
            // We only want to change the selection if its different
            if (!selectionType.equals(this.selectionType) || !selectionNodeString.equals(this.selectionNodeString)){
                if (log.isTraceEnabled()) { log.trace("Setting new selection."); }
                this.selectionType = selectionType;
                this.selectionNodeString = selectionNodeString;
                this.selectionId = selectionId;
                this.selectionEntityPath = new EntityPath(commonService, selectionNodeString);
                try {
                    Class clazz = Class.forName(this.selectionType);
                    if (log.isTraceEnabled()) { log.trace("clazz: " + clazz); }
                    this.currentSelection = commonService.get(clazz, Long.valueOf(selectionId));
                } catch (ClassNotFoundException cnfe){
                    throw new RuntimeException("Invalid type selected: " + this.selectionType, cnfe);
                } catch (NumberFormatException nfe){
                    throw new RuntimeException("selectionId is non-numeric: " + selectionId, nfe);
                }
            } else {
                if (log.isTraceEnabled()) { log.trace("Selection has remained the same, as type and ID are identical."); }
            }
        }
        if (log.isInfoEnabled()) { log.info("RETURNING currentSelection: " + currentSelection); }
        return currentSelection;
    }
    
    public EntityPath getSelectionEntityPath(){
        return this.selectionEntityPath;
    }

    public void setCurrentSelection(Object currentSelection) {
        if (log.isInfoEnabled()) { log.info("BEGIN setSelection: " + currentSelection); }
        this.currentSelection = currentSelection;
    }


    public String getSelectionNodeString() {
        return selectionNodeString;
    }


    public void setSelectionNodeString(String selectionNodeString) {
        this.selectionNodeString = selectionNodeString;
    }


    public void setSelectionType(String selectionType) {
        this.selectionType = selectionType;
    }


    public CommonService getCommonService() {
        return commonService;
    }

    public void setCommonService(CommonService commonService) {
        this.commonService = commonService;
    }

}
